<?php




/*
* T_ML_COMMENT does not exist in PHP 5.
* The following three lines define it in order to
* preserve backwards compatibility.
*
* The next two lines define the PHP 5 only T_DOC_COMMENT,
* which we will mask as T_ML_COMMENT for PHP 4.
*/
if (!defined('T_ML_COMMENT')) {
   define('T_ML_COMMENT', T_COMMENT);
} else {
   define('T_DOC_COMMENT', T_ML_COMMENT);
}


/**
 * Info about a class
 * @package ClassProfiler
 * @author Stephen Klancher <smknight@gmail.com>
 */
class clsProfile
{
	protected $content;
	protected $reflection;
	public $parseOutput;
	public $classname;
	
	public function __construct($classname)
	{
		$this->classname=$classname;
		
		//use reflection on the class
		$this->reflection = new ReflectionClass($classname);
		
		//find the file and range of lines of the class from reflection
		//	then read those range of lines to get the source
		$source=self::readLine($this->reflection->getFileName(),$this->reflection->getStartLine(),$this->reflection->getEndLine());
		
		//create the content class from the source
		$this->content=new clsContent($source,$classname);
		
		//parse the source
		$this->parseOutput=$this->content->parse();
		
		//use reflection to
		
		//add references to other classes in type hinted parameters
		$this->addTypeHintedParameters();
		
		//add parent class if any
		$this->addParentClass();
	}
	
	
	
	public function other_class_list()
	{
		return $this->content->other_class_list();
	}
	
	
	
	/**
	* Uses reflection to add type hinted parameter references to $this->content->external
	*/
	public function addTypeHintedParameters()
	{
		foreach($this->reflection->getMethods() as $m){
			foreach($m->getParameters() as $p){
				if($p->getClass()){
					$this->content->addExternalCall($p->getClass()->getName(),'Type Hinted Parameter');
				}
			}
		}
	}
	
	/**
	* Add any parent class
	*/
	public function addParentClass()
	{
		//could loop and get all ancestors, but I don't think I want that right now
		if($this->reflection->getParentClass()){
			$this->content->addExternalCall($this->reflection->getParentClass()->getName(),'Inherits');
		}
	}
	
	
	/**
	* Returns a table of external calls from this class
	* @return string html table
	*/
	public function displayExternalCalls()
	{
		$exClasses=$this->content->external;
		
		$out='<h1>'.$this->classname.' uses...</h1><table border="1" rules="rows">';
		
		//for each external class used
		foreach($exClasses as $class){
			$out.='<tr><td><h2>'.$class->name .'('.$class->total(). ')</h2>';
			$out.='<table border="1">';
			
			//for each call (instantiation, function, variable, constant) to this
			$calls=$class->items();
			foreach($calls as $call => $timesCalled){
				$out.='<tr><td>'.$call.'</td><td>'.$timesCalled.'</td></tr>';
			}
			$out.='</table>';
		}
		
		$out.='</table>';
		
		return $out;
	}
	
	
	/* This is the static comparing function: */
    static function compareClassConnections(clsProfile $a, clsProfile $b)
    {
        $al = $a->content->numberOfClassConnections();
        $bl = $b->content->numberOfClassConnections();
        if ($al == $bl) {
            return 0;
        }
        return ($al < $bl) ? +1 : -1;
    }
	
	
	/**
	*
	* Read a range of line numbers from a file
	*
	* Modified from http://www.phpro.org/examples/Read-Line-From-File.html
	*
	* @param    string    $file    The path to the file
	* @param    int    $line_start    The line number to start
	* @param    int    $line_end  The line number to end
	* @param    string    $delimiter    The character that delimits lines
	* @return    string    The line that is read
	*
	*/
	public static function readLine($file, $line_start, $line_end, $delimiter="\n")
	{
		/*** set the counter to one ***/
		$i = 1;
		
		/*** open the file for reading ***/
		$fp = fopen( $file, 'r' );
		
		//what we'll return
		$out='';
		
		/*** loop over the file pointer ***/
		while ( !feof ( $fp) )
		{
			/*** read the line into a buffer ***/
			$buffer = stream_get_line( $fp, 1024, $delimiter );
			/*** if we are at the right line number ***/
			if( $i >= $line_start && $i <=$line_end )
			{
				/*** return the line that is currently in the buffer ***/
				$out.=$buffer . $delimiter;
			}
			
			if($i > $line_end){
				return $out;
			}
			
			/*** increment the line counter ***/
			$i++;
			/*** clear the buffer ***/
			$buffer = '';
		}
		return $out;
	}
}



/**
 * Source of a class and info from it
 * @package ClassProfiler
 * @author Stephen Klancher <smknight@gmail.com>
 */
class clsContent
{
	private $clsSource;
	public $classname;
	
	//array of clsOther
	public $external=array();
	
	
	
	public function __construct($clsSource,$classname)
	{
		$this->clsSource=$clsSource;
		$this->classname=$classname;
	}
	
	/**
	* Adds an external call, creating a clsOther if needed
	*/
	public function addExternalCall($class, $call)
	{
		//change [classname]::calls to self::calls
		if($class==$this->classname){
			$class='self';
			//...or instead, just not use list self
			return false;
		}
		
		if($this->external[$class]===NULL){
			$this->external[$class]=new clsOther($class);
		}
		$this->external[$class]->add($call);
	}
	
	//number of other classes this class is connected to
	public function numberOfClassConnections()
	{
		$number=0;
		
		foreach($this->external as $class){
			$number+=$class->total();
		}
		
		return $number;
	}
	
	//returns classname => callsTo
	public function other_class_list()
	{
		$list=array();
		
		foreach($this->external as $class){
			$list[$class->name]=$class->total();
		}
		
		//highest number of connections first
		arsort($list);
		
		return $list;
	}
	
	public function parse()
	{
		$this->external=array();
		
		$tokens=token_get_all('<?php' . $this->clsSource . '?>');
		
		$out='';
		
		$prevTok=array(0,'',0);
		$isNewInstance=false;
		$isStaicCall=false;
		$staticClass='';
		
		//var_dump($tokens);
		
		foreach ($tokens as $token) {
			if (is_string($token)) {
				// simple 1-character token
				$out.=$token;
			} else {
				// token array
				$id=$token[0];
				$text=$token[1];
				$line=$token[2];
				
				
				switch ($id) {
					case T_WHITESPACE:
						$out.=$text;
						break;
					case T_COMMENT: 
					case T_ML_COMMENT: 
					case T_DOC_COMMENT: 
						break;
					
					case T_DOUBLE_COLON;
						$isStaicCall=true;
						$staticClass=$prevTok[1];
						$out.=htmlentities($text).'{'.token_name($id).'}';
						break;
					case T_NEW:
						$isNewInstance=true;
						$out.=htmlentities($text).'{'.token_name($id).'}';
						break;
						
					default:
						//looking for the class after a "new"
						if($isNewInstance){
							$isNewInstance=false;
							$this->addExternalCall($text,'new');
							continue; //skip the switch statment
						}
						
						//looking for the method/variable/constant after ::
						if($isStaicCall){
							$isStaicCall=false;
							$this->addExternalCall($staticClass,$text);
							$staticClass='';
							continue; //skip the switch statment
						}
						
						
						// anything else -> output "as is"
						$out.=htmlentities($text).'{'.token_name($id).'}';
						break;
				}
			}
			
			//keep previous token
			$prevTok=$token;
		}
		
		uasort($this->external,array('clsOther','compareTotalCalls'));
		
		return $out;
	}
	
	
}


/**
 * Represents the collection of calls or instantiations of another class
 * @package ClassProfiler
 * @author Stephen Klancher <smknight@gmail.com>
 */
class clsOther
{
	public $name;
	private $calls=array();
	
	public function __construct($name)
	{
		$this->name=$name;
	}
	
	public function add($call)
	{
		if($this->calls[$call]===null){
			$this->calls[$call]=1;
		}else{
			$this->calls[$call]+=1;
		}
	}
	
	public function items()
	{
		arsort($this->calls);
		return $this->calls;
	}
	
	public function total()
	{
		return count($this->calls);
	}
	
	
	/* This is the static comparing function: */
    static function compareTotalCalls(clsOther $a, clsOther $b)
    {
        $al = $a->total();
        $bl = $b->total();
        if ($al == $bl) {
            return 0;
        }
        return ($al < $bl) ? +1 : -1;
    }
	
}

/**
 * Represents a set of classes to be mapped
 * @package ClassProfiler
 * @author Stephen Klancher <smknight@gmail.com>
 */
class clsMap
{
	public $classes=array();
	
	public function __construct(array $classesToAdd)
	{
		$this->add($classesToAdd);
	}
	
	public function add(array $classesToAdd)
	{
		foreach($classesToAdd as $class){
			$this->classes[]=new clsProfile($class);
		}
	}
	
	public function display()
	{
		uasort($this->classes,array('clsProfile','compareClassConnections'));
		foreach($this->classes as $class){
			$out.=$class->displayExternalCalls();
		}
		return $out;
	}
	
	public static function mapSelf()
	{
		return new clsMap(array('clsMap','clsProfile','clsContent','clsOther'));
	}
	
}



//$map=new clsMap(array('MovieApp','auth','Media','MediaList','Watched','settings','User','Group','IMDb','IMDbParse','smk','Metacritic','EpisodeList','js','Movie'));
//print($map->display());


?>
