<?php
require_once 'class_clsProfile.php';
/**
 * Draw a simple diagram
 * @package ClassProfiler
 * @author Stephen Klancher <smknight@gmail.com>
 */
class Diagram
{
	const DIST_BETWEEN=150;
	const CENTER=650;
	//matrix modifier of coordinates from center
	protected $x_mult=array(1,1,0,-1,-1,-1,0,1);
	protected $y_mult=array(0,1,1,1,0,-1,-1,-1);
	
	protected $image;
	
	protected $obj=array();
	
	protected $white;
	
	public function __construct()
	{
		$this->image= @imagecreatetruecolor(self::CENTER * 2,self::CENTER * 2)
			  or die('Cannot Initialize new GD image stream');
		
		$this->white=$this->createcolor($this->image,255,255,255);
		
	}
	
	public function add(DiagramObject $obj)
	{
		//set coords if not set
		if($obj->x==null){
			$obj->set_coords($this->next_coord_spiral());
		}
		
		//set color if not set
		if($obj->color==null){
			$obj->color=$this->random_color();
		}
		
		//add
		$this->obj[]=$obj;
	}
	
	
	public function draw()
	{
		//two loops might be slower but should keep from connections being drawn on top of items
		
		//draw connections
		$i=0;
		foreach($this->obj as $o){
			$i+=1;
			$o->draw_connections($this->image);
		}
		
		//draw items
		$i=0;
		foreach($this->obj as $o){
			$i+=1;
			
			$x_bound=$o->x+ (imagefontwidth(5) * strlen($o));
			$y_bound=$o->y+ imagefontheight(5);
			
			imagefilledrectangle ($this->image,$o->x,$o->y,$x_bound,$y_bound,$o->color);
			imagestring($this->image, 5, $o->x, $o->y,  $o, $this->white);
		}
		
	}
	
	public function list_coords()
	{
		foreach($this->obj as $o){
			print($o);
		}
	}
	
	
	public function write_png($file)
	{
		$im = $this->image;
		
		imagepng($im,$file);
		return '<img src="'.$file.'">';
	}
	
	
	public function random_color()
	{
		//assign random rgb values
		$c1 = mt_rand(50,200); //r(ed)
		$c2 = mt_rand(50,200); //g(reen)
		$c3 = mt_rand(50,200); //b(lue)
		return $this->createcolor($this->image,$c1,$c2,$c3);
	}
	
	//http://us2.php.net/manual/en/function.imagecolorallocate.php#94785
	public function createcolor($pic,$c1,$c2,$c3) {
		//get color from palette
		$color = imagecolorexact($pic, $c1, $c2, $c3);
		if($color==-1) {
			//color does not exist...
			//test if we have used up palette
			if(imagecolorstotal($pic)>=255) {
				//palette used up; pick closest assigned color
				$color = imagecolorclosest($pic, $c1, $c2, $c3);
			} else {
				//palette NOT used up; assign new color
				$color = imagecolorallocate($pic, $c1, $c2, $c3);
			}
		}
		return $color;
	}
	
	protected function next_coord_spiral()
	{
		$element_number=count($this->obj);
		if($element_number==0){
			//first item in center
			return array('x'=>self::CENTER,'y'=>self::CENTER);
		}else{
			$pos_around=($element_number-1) % 8 +1; //1
			$x_mult=$this->x_mult[$pos_around-1]; //1
			$y_mult=$this->y_mult[$pos_around-1]; //1
			$increments_from_center=floor(($element_number-1) / 8) + 1;
			$cur_dist_from_center=$increments_from_center * self::DIST_BETWEEN;
			$x_from_center=$x_mult * $cur_dist_from_center;
			$y_from_center=$y_mult * $cur_dist_from_center;
			
			$x=self::CENTER + $x_from_center + mt_rand(-30,30);
			$y=self::CENTER + $y_from_center + mt_rand(-30,30);
			
			return array('x'=>$x,'y'=>$y);
		}
	}
	
	public function testspiral()
	{
		for($i=1;$i<=20;$i++){
			$element_number=$i;
			print('$element_number:'.$element_number);
			$pos_around=($element_number-1) % 8 +1; //1
			print(',   $pos_around:'.$pos_around);
			$x_mult=$this->x_mult[$pos_around-1]; //1
			$y_mult=$this->y_mult[$pos_around-1]; //1
			print(',   mult:'.$x_mult . ','.$y_mult);
			
			$increments_from_center=floor(($element_number-1) / 8) + 1;
			print(',   $increments_from_center:'.$increments_from_center);
			$cur_dist_from_center=$increments_from_center * self::DIST_BETWEEN;
			print(',   $cur_dist_from_center:'.$cur_dist_from_center);
			$x_from_center=$x_mult * $cur_dist_from_center;
			$y_from_center=$y_mult * $cur_dist_from_center;
			print(',   from_center:'.$x_from_center.','.$y_from_center);
			$x=self::CENTER + $x_from_center;
			$y=self::CENTER + $y_from_center;
			print(',   coords:'.$x.','.$y);
			print("\n\n");
		}
	}
	
}

class ClassDiagram extends Diagram
{
	public function add(clsProfile $class)
	{
		//create the diagram object from a class
		$obj=new ClassDiagramObject($class);
		//add via parent
		parent::add($obj);
	}
	
	public function draw()
	{
		//generate connections
		$this->create_connections();
		//draw via parent
		parent::draw();
	}
	
	public function create_connections()
	{
		//for each class diagram object
		$classlist=$this->obj;
		foreach($classlist as $c){
			//for each other class it mentiones
			$list=$c->other_class_list();
			foreach($list as $name=>$connections){
				//check each class
				foreach($this->obj as $oc){
					//if the name matches
					if($name==$oc->class->classname){
						//add connection
						$con=new ConnectionLine($c->x +(($c->connection_count()+1) * 10),
												$c->y + 20,$oc->x+10, $oc->y-10,$connections,'');
						$c->add_connection($con);
						break; //done looking for this other class
					}
				}
			}
		}
	}
}

class DiagramObject
{
	public $x;
	public $y;
	public $color;
	protected $connections=array();
	
	
	public function __construct()
	{
		
	}
	
	public function connection_count()
	{
		return count($this->connections);
	}
	
	
	public function set_coords(array $coords)
	{
		if(array_key_exists('x',$coords)){
			$this->x=$coords['x'];
			$this->y=$coords['y'];
		}else{
			$this->x=$coords[0];
			$this->y=$coords[1];
		}
	}
	
	public function draw_connections($image)
	{
		foreach($this->connections as $c){
			$c->draw_line($image, $this->color);
		}
	}
	
	
	public function add_connection(ConnectionLine $con)
	{
		$this->connections[]=$con;
	}
	
	
	
	public function __toString()
	{
		return $this->x . ',' . $this->y;
	}
	
}

class ClassDiagramObject extends DiagramObject
{
	public $class;
	
	
	public function __construct(clsProfile $class)
	{
		$this->class=$class;
	}
	
	
	
	
	public function other_class_list()
	{
		return $this->class->other_class_list();
	}
	
	
	
	public function __toString()
	{
		return $this->class->classname; //. ' (' . $this->x . ',' . $this->y . ")";

	}
}


class ConnectionLine
{
	protected $start_x;
	protected $start_y;
	protected $end_x;
	protected $end_y;
	protected $start_txt;
	protected $end_txt;
	
	public function __construct($start_x,$start_y,$end_x,$end_y,$start_txt,$end_txt)
	{
		$this->start_x=$start_x;
		$this->start_y=$start_y;
		$this->start_txt=$start_txt;
		$this->end_x=$end_x;
		$this->end_y=$end_y;
		$this->end_txt=$end_txt;
	}
	
	public function draw_line($image,$color)
	{
		//line/line with arrow
		//imageline($image,$this->start_x,$this->start_y,$this->end_x,$this->end_y,$color);
		$this->arrow($image,$this->start_x,$this->start_y,$this->end_x,$this->end_y,10,4,$color);
		//start text
		imagestring($image,2,$this->start_x,$this->start_y,$this->start_txt,$color);
		//end text
		imagestring($image,2,$this->end_x,$this->end_y,$this->end_txt,$color);
	}
	
	//http://www.php.net/manual/en/function.imageline.php#92168
	public function arrow($im, $x1, $y1, $x2, $y2, $alength, $awidth, $color) {
		$distance = sqrt(pow($x1 - $x2, 2) + pow($y1 - $y2, 2));
	
		$dx = $x2 + ($x1 - $x2) * $alength / $distance;
		$dy = $y2 + ($y1 - $y2) * $alength / $distance;
	
		$k = $awidth / $alength;
	
		$x2o = $x2 - $dx;
		$y2o = $dy - $y2;
	
		$x3 = $y2o * $k + $dx;
		$y3 = $x2o * $k + $dy;
	
		$x4 = $dx - $y2o * $k;
		$y4 = $dy - $x2o * $k;
	
		imageline($im, $x1, $y1, $dx, $dy, $color);
		imagefilledpolygon($im, array($x2, $y2, $x3, $y3, $x4, $y4), 3, $color);
	}
}



?>